@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[146]:
+# In[42]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[147]:
+# In[43]:
 
 
-version = '0.69'
+version = '0.72'
 
 import re
 import tempfile
@@ -33,7 +33,7 @@ import time
 
 # # Helper Functions
 
-# In[148]:
+# In[44]:
 
 
 def is_testing():
@@ -43,7 +43,7 @@ def is_testing():
         return False
 
 
-# In[149]:
+# In[45]:
 
 
 def isnotebook():
@@ -54,7 +54,7 @@ def isnotebook():
     return False
 
 
-# In[150]:
+# In[46]:
 
 
 def testPrint(*args):
@@ -64,13 +64,13 @@ def testPrint(*args):
 
 # # Timing
 
-# In[151]:
+# In[47]:
 
 
 timeSinceLastCallDict = {}
 
 
-# In[152]:
+# In[48]:
 
 
 def timeSinceLastCall(name):
@@ -83,7 +83,7 @@ def timeSinceLastCall(name):
     return name + ": " + time_s
 
 
-# In[153]:
+# In[49]:
 
 
 if is_testing():
@@ -92,7 +92,7 @@ if is_testing():
 
 # # Requiremtns
 
-# In[154]:
+# In[50]:
 
 
 requirements = []
@@ -124,7 +124,7 @@ import yaml
 
 # # Strings
 
-# In[155]:
+# In[51]:
 
 
 def sFloat0(f): return "{:0.0f}".format(f)
@@ -154,7 +154,7 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
-# In[156]:
+# In[52]:
 
 
 def strUuid4():
@@ -165,14 +165,14 @@ def strUuid4():
     return uuid
 
 
-# In[157]:
+# In[53]:
 
 
 if is_testing():
     print(strUuid4())
 
 
-# In[158]:
+# In[54]:
 
 
 def strRand(n, num=True, upperCase=True, lowerCase=True):
@@ -184,7 +184,7 @@ def strRand(n, num=True, upperCase=True, lowerCase=True):
     return ''.join(random.choice(pool) for _ in range(n))
 
 
-# In[159]:
+# In[55]:
 
 
 if is_testing():
@@ -193,7 +193,7 @@ if is_testing():
     print(strRand(20, upperCase=False, lowerCase=False))
 
 
-# In[160]:
+# In[56]:
 
 
 def strMd5Pickle4(obj):
@@ -203,14 +203,14 @@ def strMd5Pickle4(obj):
     return md5.hexdigest()
 
 
-# In[161]:
+# In[57]:
 
 
 if is_testing():
     print(strMd5Pickle4(['a', 'b']))
 
 
-# In[162]:
+# In[58]:
 
 
 def strMd5String(s):
@@ -219,28 +219,30 @@ def strMd5String(s):
     return md5.hexdigest()
 
 
-# In[163]:
+# In[59]:
 
 
 if is_testing():
     print(strMd5String('s'))
 
 
-# # Process / Thread
+# # Multiprocess / Thread
 
-# In[164]:
+# In[60]:
 
 
 def mapThread(func, args, n=8):
     from multiprocessing.pool import ThreadPool
     tPool = ThreadPool(n)
     res = tPool.map(func, args)
+    tPool.close()
     return res
 
 def mapProcess(func, args, n=8):
     from multiprocessing import Pool
     pPool = Pool(n)
     res = pPool.map(func, args)
+    pPool.close()
     return res
 
 if is_testing():
@@ -251,14 +253,14 @@ if is_testing():
 
 # # Lists
 
-# In[165]:
+# In[61]:
 
 
 def liChunks(l, chunk_size):
     return [l[i:i + chunk_size] for i in range(0, len(l), chunk_size)]
 
 
-# In[166]:
+# In[62]:
 
 
 if is_testing():
@@ -267,7 +269,7 @@ if is_testing():
     print(liChunks(['a', 'b', 'c', 'd', 'e', 'f'], 3))
 
 
-# In[167]:
+# In[63]:
 
 
 def liSplit(a, number_of_splits):
@@ -275,7 +277,7 @@ def liSplit(a, number_of_splits):
     return [a[start:end] for start, end in zip(idxs[:-1], idxs[1:])]
 
 
-# In[168]:
+# In[64]:
 
 
 if is_testing():
@@ -287,7 +289,7 @@ if is_testing():
     print(liSplit(['a', 'b', 'c', 'd', 'e'], 6))
 
 
-# In[169]:
+# In[65]:
 
 
 def liJoin(l):
@@ -297,14 +299,14 @@ def liJoin(l):
     return out
 
 
-# In[170]:
+# In[66]:
 
 
 if is_testing():
     print(liJoin([['a', 'b', 'c'], ['d', 'e', 'f']]))
 
 
-# In[171]:
+# In[67]:
 
 
 def liFlatten(l):
@@ -321,7 +323,7 @@ def liFlatten(l):
         return l
 
 
-# In[172]:
+# In[68]:
 
 
 if is_testing():
@@ -331,7 +333,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[173]:
+# In[69]:
 
 
 regex_cache = {}
@@ -341,35 +343,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[174]:
+# In[70]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[175]:
+# In[71]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[176]:
+# In[72]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[177]:
+# In[73]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[178]:
+# In[74]:
 
 
 def lCsv(l, force_oneline=False):
@@ -385,21 +387,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[179]:
+# In[75]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[180]:
+# In[76]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[181]:
+# In[77]:
 
 
 if is_testing():
@@ -408,7 +410,7 @@ if is_testing():
 
 # # Text files
 
-# In[182]:
+# In[78]:
 
 
 def txtread(path):
@@ -429,7 +431,7 @@ def txtwrite(path, txt):
             return f.write(txt)
 
 
-# In[183]:
+# In[79]:
 
 
 if is_testing():
@@ -438,21 +440,21 @@ if is_testing():
 
 # # YAML files
 
-# In[184]:
+# In[80]:
 
 
 def yamlwrite(path, obj):
     txtwrite(path, yaml.dump(obj))
 
 
-# In[185]:
+# In[81]:
 
 
 def yamlread(path):
     return yaml.safe_load(txtread(path=path))
 
 
-# In[186]:
+# In[82]:
 
 
 if is_testing():
@@ -465,7 +467,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[187]:
+# In[83]:
 
 
 regex_cache = {}
@@ -477,35 +479,35 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[188]:
+# In[84]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[189]:
+# In[85]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[190]:
+# In[86]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[191]:
+# In[87]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[192]:
+# In[88]:
 
 
 if is_testing():
@@ -514,7 +516,7 @@ if is_testing():
 
 # # GCP Google Cloud Platform
 
-# In[193]:
+# In[89]:
 
 
 import warnings
@@ -538,7 +540,7 @@ def _gsBlob(path):
     return blob
 
 
-# In[194]:
+# In[90]:
 
 
 def gsGsToHttp(path):
@@ -548,7 +550,7 @@ def gsHttpToGs(path):
     return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
 
 
-# In[195]:
+# In[91]:
 
 
 def gsLs(path):
@@ -558,7 +560,7 @@ def gsLs(path):
     return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
 
 
-# In[196]:
+# In[92]:
 
 
 def gsExists(path):
@@ -568,7 +570,7 @@ def gsExists(path):
     return client.blob(prefix).exists()
 
 
-# In[197]:
+# In[93]:
 
 
 def gsUploadFile(gsPath, path):
@@ -576,7 +578,7 @@ def gsUploadFile(gsPath, path):
     blob.upload_from_filename(path)
 
 
-# In[198]:
+# In[94]:
 
 
 def gsTxtread(path):
@@ -588,7 +590,7 @@ def gsTxtread(path):
     return blob.download_as_string().decode()
 
 
-# In[199]:
+# In[95]:
 
 
 def gsTxtwrite(path, s):
@@ -596,7 +598,7 @@ def gsTxtwrite(path, s):
     return blob.upload_from_string(s)
 
 
-# In[200]:
+# In[96]:
 
 
 def gsImwrite(path, img):
@@ -605,7 +607,7 @@ def gsImwrite(path, img):
     blob.upload_from_string(encimg.tobytes())
 
 
-# In[201]:
+# In[97]:
 
 
 def gsImread(path):
@@ -614,7 +616,7 @@ def gsImread(path):
     return cv2.imdecode(raw, 1)[:, :, [2, 1, 0]]
 
 
-# In[202]:
+# In[98]:
 
 
 def gsPklWrite(path, obj):
@@ -622,7 +624,7 @@ def gsPklWrite(path, obj):
     return blob.upload_from_string(pickle.dumps(obj, protocol=4))
 
 
-# In[203]:
+# In[99]:
 
 
 def gsPklRead(path):
@@ -632,18 +634,20 @@ def gsPklRead(path):
 
 # # Pickle
 
-# In[204]:
+# In[169]:
 
 
 def pklwrite(path, obj):
     if 'gs://' in path:
         return gsPklWrite(path, obj)
     else:
+        if len(os.path.dirname(path)) > 0:
+            os.makedirs(os.path.dirname(path), exist_ok=True)
         with open(path, 'wb') as f:
             return pickle.dump(obj, f, protocol=4)
 
 
-# In[205]:
+# In[101]:
 
 
 def pklread(path):
@@ -654,7 +658,7 @@ def pklread(path):
             return pickle.load(f)
 
 
-# In[206]:
+# In[102]:
 
 
 if is_testing():
@@ -667,7 +671,7 @@ if is_testing():
 
 # # Images
 
-# In[207]:
+# In[103]:
 
 
 def imread(path):
@@ -701,7 +705,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[208]:
+# In[104]:
 
 
 def imshow(array, scale=1):
@@ -716,7 +720,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[209]:
+# In[105]:
 
 
 if is_testing():
@@ -724,14 +728,14 @@ if is_testing():
     imshow(img)
 
 
-# In[210]:
+# In[106]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[211]:
+# In[107]:
 
 
 def imscaleNN(img, s):
@@ -741,7 +745,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[212]:
+# In[108]:
 
 
 if is_testing():
@@ -760,7 +764,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[213]:
+# In[109]:
 
 
 def imRepeat(img, height, width):
@@ -774,7 +778,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[214]:
+# In[110]:
 
 
 if is_testing():
@@ -782,33 +786,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[215]:
+# In[111]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[216]:
+# In[112]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[217]:
+# In[113]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[218]:
+# In[114]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[219]:
+# In[115]:
 
 
 def imAddNoiseGauss(img, std):
@@ -820,7 +824,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[220]:
+# In[116]:
 
 
 def imJpgDegradation(img, quality):
@@ -833,7 +837,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[221]:
+# In[117]:
 
 
 def imCropCenter(img, size):
@@ -848,35 +852,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[222]:
+# In[118]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[223]:
+# In[119]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[224]:
+# In[120]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[225]:
+# In[121]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[226]:
+# In[122]:
 
 
 def imCropRandom(img, size):
@@ -891,7 +895,7 @@ def imCropRandom(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[227]:
+# In[123]:
 
 
 if is_testing(): 
@@ -899,7 +903,7 @@ if is_testing():
     imshow(imCropRandom(imread('http://via.placeholder.com/40.png'), 39))
 
 
-# In[228]:
+# In[124]:
 
 
 def imGallery(imgs, pad=0):
@@ -930,7 +934,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[229]:
+# In[125]:
 
 
 if is_testing():
@@ -938,49 +942,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[230]:
+# In[126]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[231]:
+# In[127]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[232]:
+# In[128]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[233]:
+# In[129]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[234]:
+# In[130]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[235]:
+# In[131]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[236]:
+# In[132]:
 
 
 if is_testing():
@@ -990,14 +994,14 @@ if is_testing():
 
 # # Files
 
-# In[237]:
+# In[133]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[238]:
+# In[134]:
 
 
 if is_testing():
@@ -1026,7 +1030,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[239]:
+# In[135]:
 
 
 def fiRemove(path):
@@ -1054,7 +1058,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[240]:
+# In[136]:
 
 
 if is_testing():
@@ -1086,7 +1090,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[241]:
+# In[137]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -1101,7 +1105,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[242]:
+# In[138]:
 
 
 def fiJoin(*args):
@@ -1113,7 +1117,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[243]:
+# In[139]:
 
 
 def jnNbFullWidth():
@@ -1127,7 +1131,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[244]:
+# In[140]:
 
 
 def jnImageRenderingPixelated():
@@ -1139,7 +1143,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[245]:
+# In[141]:
 
 
 def jnDefault():
@@ -1157,7 +1161,7 @@ if is_testing():
 
 
 
-# In[246]:
+# In[142]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1170,7 +1174,7 @@ if is_testing():
     
 
 
-# In[247]:
+# In[143]:
 
 
 def jnImgToHtmlB64(img):
@@ -1205,7 +1209,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[248]:
+# In[144]:
 
 
 if is_testing():
@@ -1213,7 +1217,7 @@ if is_testing():
    
 
 
-# In[249]:
+# In[145]:
 
 
 if is_testing():
@@ -1223,7 +1227,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[250]:
+# In[146]:
 
 
 
@@ -1233,7 +1237,7 @@ if is_testing():
 
 # # Shell
 
-# In[251]:
+# In[147]:
 
 
 def shell(cmd):
@@ -1248,7 +1252,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[252]:
+# In[148]:
 
 
 def gitGetHash():
@@ -1258,7 +1262,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[253]:
+# In[149]:
 
 
 def gitGetBranch():
@@ -1268,7 +1272,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[254]:
+# In[150]:
 
 
 def gitGetMessage():
@@ -1278,7 +1282,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[255]:
+# In[151]:
 
 
 def gitGetAllFiles():
@@ -1288,7 +1292,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[256]:
+# In[152]:
 
 
 def gitListByTime(path=None):
@@ -1304,18 +1308,29 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[257]:
+# In[153]:
 
 
 def gitTime(path):
-    _time = lambda x: shell('git log -1 --format="%ad" -- {filename}'.format(filename=x))
+    _time = lambda x: shell('git log -1 --format="%ad" -- {filename}'.format(filename=x)).replace("\n", "")
     return _time(path)
     
 if is_testing():
     print(gitTime("__init__.py"))
 
 
-# In[258]:
+# In[154]:
+
+
+def gitTimeRaw(path):
+    _time = lambda x: shell('git log -1 --format="%ad" --date=raw -- {filename}'.format(filename=x)).replace("\n", "")
+    return _time(path)
+    
+if is_testing():
+    print(gitTimeRaw("__init__.py"))
+
+
+# In[155]:
 
 
 def shellSortDirsBySize():
@@ -1327,14 +1342,14 @@ if is_testing():
 
 # # Package Files
 
-# In[259]:
+# In[156]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[260]:
+# In[157]:
 
 
 def shell(cmd):
@@ -1342,7 +1357,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[261]:
+# In[158]:
 
 
 if is_testing():
@@ -1374,7 +1389,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[262]:
+# In[159]:
 
 
 if is_testing():
@@ -1384,7 +1399,7 @@ description-file = README.md
 """
 
 
-# In[263]:
+# In[160]:
 
 
 if is_testing():
@@ -1407,14 +1422,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[264]:
+# In[161]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[265]:
+# In[162]:
 
 
 if is_testing():
@@ -1423,7 +1438,7 @@ if is_testing():
     
 
 
-# In[266]:
+# In[163]:
 
 
 if is_testing():
@@ -1434,7 +1449,7 @@ lx/__init__.py
 """
 
 
-# In[267]:
+# In[164]:
 
 
 if is_testing():
